Index: src/http/modules/perl/ngx_http_perl_module.c
===================================================================
--- src/http/modules/perl/ngx_http_perl_module.c	(revision 4526)
+++ src/http/modules/perl/ngx_http_perl_module.c	(working copy)
@@ -44,7 +44,8 @@
     ngx_log_t *log);
 static ngx_int_t ngx_http_perl_call_handler(pTHX_ ngx_http_request_t *r,
     HV *nginx, SV *sub, SV **args, ngx_str_t *handler, ngx_str_t *rv);
-static void ngx_http_perl_eval_anon_sub(pTHX_ ngx_str_t *handler, SV **sv);
+static void ngx_http_perl_eval_anon_sub(pTHX_ ngx_pool_t *pool,
+    ngx_str_t *handler, SV **sv);
 
 static ngx_int_t ngx_http_perl_preconfiguration(ngx_conf_t *cf);
 static void *ngx_http_perl_create_main_conf(ngx_conf_t *cf);
@@ -381,7 +382,6 @@
     ctx->ssi = ssi_ctx;
 
     handler = params[NGX_HTTP_PERL_SSI_SUB];
-    handler->data[handler->len] = '\0';
 
     {
 
@@ -392,7 +392,7 @@
 
     /* the code is disabled to force the precompiled perl code using only */
 
-    ngx_http_perl_eval_anon_sub(aTHX_ handler, &sv);
+    ngx_http_perl_eval_anon_sub(aTHX_ r->pool, handler, &sv);
 
     if (sv == &PL_sv_undef) {
         ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
@@ -762,22 +762,36 @@
 
 
 static void
-ngx_http_perl_eval_anon_sub(pTHX_ ngx_str_t *handler, SV **sv)
+ngx_http_perl_eval_anon_sub(pTHX_ ngx_pool_t *pool, ngx_str_t *handler,
+    SV **sv)
 {
-    u_char  *p;
+    u_char  *p, *buf;
+    size_t  len;
 
-    for (p = handler->data; *p; p++) {
+    for (p = handler->data; p < handler->data + handler->len; p++) {
         if (*p != ' ' && *p != '\t' && *p != CR && *p != LF) {
             break;
         }
     }
 
-    if (ngx_strncmp(p, "sub ", 4) == 0
-        || ngx_strncmp(p, "sub{", 4) == 0
-        || ngx_strncmp(p, "use ", 4) == 0)
+    len = handler->data + handler->len - p;
+
+    if (len >= 4 &&
+        (ngx_memcmp(p, "sub ", 4) == 0
+        || ngx_memcmp(p, "sub{", 4) == 0
+        || ngx_memcmp(p, "use ", 4) == 0))
     {
-        *sv = eval_pv((char *) p, FALSE);
+        buf = ngx_palloc(pool, len + 1);
+        if (buf == NULL) {
+            *sv = NULL;
+            return;
+        }
 
+        ngx_memcpy(buf, p, len);
+        buf[len] = '\0';
+
+        *sv = eval_pv((char *) buf, FALSE);
+
         /* eval_pv() does not set ERRSV on failure */
 
         return;
@@ -936,7 +950,7 @@
     dTHXa(pmcf->perl);
     PERL_SET_CONTEXT(pmcf->perl);
 
-    ngx_http_perl_eval_anon_sub(aTHX_ &value[1], &plcf->sub);
+    ngx_http_perl_eval_anon_sub(aTHX_ cf->pool, &value[1], &plcf->sub);
 
     if (plcf->sub == &PL_sv_undef) {
         ngx_conf_log_error(NGX_LOG_ERR, cf, 0,
@@ -1007,7 +1021,7 @@
     dTHXa(pmcf->perl);
     PERL_SET_CONTEXT(pmcf->perl);
 
-    ngx_http_perl_eval_anon_sub(aTHX_ &value[2], &pv->sub);
+    ngx_http_perl_eval_anon_sub(aTHX_ cf->pool, &value[2], &pv->sub);
 
     if (pv->sub == &PL_sv_undef) {
         ngx_conf_log_error(NGX_LOG_ERR, cf, 0,